using System;
using System.Collections.Generic;
using System.Text;
using System.Windows.Documents;
using System.Windows;

namespace WpfMsdn
{
    /// <summary>
    /// Builds a WPF FlowDocument piece by piece.
    /// </summary>
    /// <remarks>This is used by the TransformToFlowDocument class to generate
    /// a FlowDocument. This adds new paragraphs, tables, Runs, Inlines, etc. as
    /// required.</remarks>
    class FlowDocBuilder
    {
        #region State

        /// <summary>
        /// The document this object is building.
        /// </summary>
        private FlowDocument m_doc;

        /// <summary>
        /// The current paragraph to which text is being appended. Null if the
        /// next piece of text should go into a new paragraph.
        /// </summary>
        private Paragraph m_currentParagraph;

        /// <summary>
        /// The run into which text is being added. (This might be a direct child
        /// of the current paragraph, or it could be inside an inline such as Bold.)
        /// </summary>
        private Run m_currentRun;

        /// <summary>
        /// The current table, if there is one. (When set, new paragraphs will be
        /// added to the last cell in the table rather than as children of the
        /// document.
        /// </summary>
        private Table m_table;

        /// <summary>
        /// Column of the next item to be added to the current table.
        /// </summary>
        /// <remarks>Used to track whether we need to add extra column
        /// definitions to the table.</remarks>
        int m_currentTableColumn;

        /// <summary>
        /// The current list, if there is one. (When set, new paragraphs will be
        /// added to the last list item rather than as children of the document.)
        /// </summary>
        private List m_list;

        #endregion

        #region Construction

        /// <summary>
        /// Constructs a new FlowDocBuilder.
        /// </summary>
        public FlowDocBuilder()
        {
            m_doc = new FlowDocument();
        }

        #endregion

        #region Properties

        /// <summary>
        /// The document this object is building.
        /// </summary>
        public FlowDocument Document
        {
            get { return m_doc; }
        }

        #endregion

        #region Text and paragraph handling

        /// <summary>
        /// Adds text to the end of the document.
        /// </summary>
        /// <param name="text">Text to add.</param>
        /// <remarks>The location in which text is added depends on the current
        /// state. If a run already exists, it will be added to that run. If not,
        /// a new one will be added, and a new paragraph will be created to hold it
        /// if necessary.</remarks>
        public void AddText(string text)
        {
            EnsureParagraphPresent();
            if (m_currentRun == null)
            {
                m_currentRun = new Run(text);
                m_currentParagraph.Inlines.Add(m_currentRun);
            }
            else
            {
                m_currentRun.Text += text;
            }
        }

        /// <summary>
        /// Ensures that the next text to be added goes into a new paragraph.
        /// </summary>
        public void PutNextTextInNewParagraph()
        {
            m_currentParagraph = null;
            m_currentRun = null;
        }

        /// <summary>
        /// Adds a line break.
        /// </summary>
        /// <remarks>This is subtly different from adding a new paragraph.
        /// Technically, the difference is that a LineBreak is just an
        /// Inline element, so it does not introduce a new paragraph into the
        /// document structure. The practical upshot is that although following
        /// text will appear on a new line, it will not have the extra space
        /// that usually appears before and after each paragraph.</remarks>
        public void LineBreak()
        {
            EnsureParagraphPresent();
            m_currentParagraph.Inlines.Add(new LineBreak());
            m_currentRun = null;
        }

        /// <summary>
        /// Add a title.
        /// </summary>
        /// <param name="heading">Title text.</param>
        public void WriteTitle(string heading)
        {
            Paragraph title = new Paragraph(new Run(heading));
            title.Style = Application.Current.FindResource("Heading") as Style;
            m_doc.Blocks.Add(title);
            PutNextTextInNewParagraph();
        }


        /// <summary>
        /// Create a paragraph if necessary.
        /// </summary>
        /// <remarks>This dynamically creates a new paragraph when
        /// required. (E.g. when text is first added either to the document or
        /// the current table cell or list item.) It will also create a new
        /// paragraph if <see cref="PutNextTextInNewParagraph"/> has been called
        /// since text was last added.</remarks>
        private void EnsureParagraphPresent()
        {
            if (m_currentParagraph == null)
            {
                m_currentParagraph = new Paragraph();
                if (m_table != null)
                {
                    TableRowCollection rows = m_table.RowGroups[0].Rows;
                    TableRow row = rows[rows.Count - 1];
                    row.Cells.Add(new TableCell(m_currentParagraph));
                }
                else if (m_list == null)
                {
                    m_doc.Blocks.Add(m_currentParagraph);
                }
                else
                {
                    m_list.ListItems.LastListItem.Blocks.Add(m_currentParagraph);
                }
            }
        }

        #endregion

        #region Inlines

        /// <summary>
        /// Starts a new Bold inline.
        /// </summary>
        /// <remarks>Further text added with <see cref="AddText"/> will be added
        /// inside this Bold element until either <see cref="EndInline"/> or
        /// <see cref="PutNextTextInNewParagraph"/> are called.</remarks>
        public void StartBold()
        {
            m_currentRun = new Run();
            Bold b = new Bold(m_currentRun);
            EnsureParagraphPresent();
            m_currentParagraph.Inlines.Add(b);
        }

        /// <summary>
        /// Starts a new Italic inline.
        /// </summary>
        /// <remarks>Further text added with <see cref="AddText"/> will be added
        /// inside this Italic element until either <see cref="EndInline"/> or
        /// <see cref="PutNextTextInNewParagraph"/> are called.</remarks>
        public void StartItalic()
        {
            m_currentRun = new Run();
            Italic b = new Italic(m_currentRun);
            EnsureParagraphPresent();
            m_currentParagraph.Inlines.Add(b);
        }

        /// <summary>
        /// Starts a new Hyperlink inline.
        /// </summary>
        /// <param name="href">Hyperlink's NavigateUri</param>
        /// <remarks>Further text added with <see cref="AddText"/> will be added
        /// inside this Hyperlink element until either <see cref="EndInline"/> or
        /// <see cref="PutNextTextInNewParagraph"/> are called.</remarks>
        public void StartHyperlink(string href)
        {
            m_currentRun = new Run();
            Hyperlink h = new Hyperlink(m_currentRun);
            h.NavigateUri = new Uri(href, UriKind.RelativeOrAbsolute);
            EnsureParagraphPresent();
            m_currentParagraph.Inlines.Add(h);
        }

        /// <summary>
        /// Finish the current inline.
        /// </summary>
        /// <remarks>Any further text added with <see cref="AddText"/> will go
        /// into a new run that follows the current inline.</remarks>
        public void EndInline()
        {
            m_currentRun = null;
        }

        #endregion

        #region Lists

        /// <summary>
        /// Start a new List
        /// </summary>
        /// <param name="markerStyle">Marker style for list elements.</param>
        /// <remarks>Adds a List block to the document. You must call
        /// <see cref="ListItem"/> for each item in the list before adding
        /// text.</remarks>
        public void StartList(TextMarkerStyle markerStyle)
        {
            m_list = new List();
            m_list.MarkerStyle = markerStyle;
            m_doc.Blocks.Add(m_list);
        }

        /// <summary>
        /// Adds a new item to the current list.
        /// </summary>
        public void ListItem()
        {
            ListItem item = new ListItem();
            m_list.ListItems.Add(item);
            PutNextTextInNewParagraph();
        }

        /// <summary>
        /// Ends the current list.
        /// </summary>
        public void EndList()
        {
            m_list = null;
            PutNextTextInNewParagraph();
        }

        #endregion

        #region Tables

        /// <summary>
        /// Add a new table.
        /// </summary>
        /// <param name="styleName">Name of table style, or ""</param>
        public void StartTable(string styleName)
        {
            m_table = new Table();
            m_table.Style = Application.Current.FindResource(styleName) as Style;
            m_table.RowGroups.Add(new TableRowGroup());
            m_doc.Blocks.Add(m_table);
            PutNextTextInNewParagraph();
        }

        /// <summary>
        /// Add a new row to the current table.
        /// </summary>
        public void AddTableRow()
        {
            TableRow row = new TableRow();
            m_table.RowGroups[0].Rows.Add(row);
            m_currentTableColumn = 0;
        }

        /// <summary>
        /// Adds a new cell to the current table row.
        /// </summary>
        public void AddTableItem()
        {
            TableRowCollection rows = m_table.RowGroups[0].Rows;
            TableRow row = rows[rows.Count - 1];
            m_currentTableColumn += 1;
            if (m_currentTableColumn > m_table.Columns.Count)
            {
                m_table.Columns.Add(new TableColumn());
            }
        }

        /// <summary>
        /// Adds a new heading cell to the table.
        /// </summary>
        public void AddTableHeaderItem()
        {
            TableRowCollection rows = m_table.RowGroups[0].Rows;
            TableRow row = rows[rows.Count - 1];
            row.Style = Application.Current.FindResource("AlertTableHeading") as Style;
            m_currentTableColumn += 1;
            if (m_currentTableColumn > m_table.Columns.Count)
            {
                m_table.Columns.Add(new TableColumn());
            }
        }

        /// <summary>
        /// Finishes the current table.
        /// </summary>
        public void EndTable()
        {
            m_table = null;
            PutNextTextInNewParagraph();
        }

        #endregion

        #region Code snippets

        /// <summary>
        /// Add a code region.
        /// </summary>
        /// <param name="snippetDisplayLanguage">Display name for language, or "".</param>
        /// <param name="content">Code snippet contents</param>
        public void WriteSnippet(string snippetDisplayLanguage, string content)
        {
            if (!string.IsNullOrEmpty(snippetDisplayLanguage))
            {
                Paragraph title = new Paragraph(new Run(snippetDisplayLanguage));
                title.Style = Application.Current.FindResource("SnippetHeading") as Style;
                m_doc.Blocks.Add(title);
                PutNextTextInNewParagraph();
            }

            Run codeRun = new Run(content);
            Paragraph snippetParagraph = new Paragraph(codeRun);
            snippetParagraph.Style = Application.Current.FindResource("CodeSnippet") as Style;
            m_doc.Blocks.Add(snippetParagraph);
            PutNextTextInNewParagraph();
        }

        #endregion

    }
}
