using System;
using System.Collections.Generic;
using System.Text;
using System.Xml.XPath;
using MsdnMan.XsltObjects;
using System.Windows.Documents;
using System.Xml;
using System.Windows;

namespace WpfMsdn
{
    /// <summary>
    /// Processes incoming XHTML from MTPS service and generates
    /// a WPF FlowDocument.
    /// </summary>
    /// <remarks>This is based on Craig's XsltObjects code. It uses template
    /// matching in the XSLT style, but running a function for each template.
    /// </remarks>
    class TransformToFlowDocument : MsdnMan.XsltObjects.Transform
    {
        public TransformToFlowDocument(XmlNameTable nameTable)
        {
            XmlNamespaceManager nsManager = new XmlNamespaceManager(nameTable);
            nsManager.AddNamespace("xhtml", "http://www.w3.org/1999/xhtml");
            nsManager.AddNamespace("mtps", "http://msdn2.microsoft.com/mtps");
            this.NamespaceResolver = nsManager;

            m_noContentYet = true;
        }

        private FlowDocBuilder m_output;
        private bool m_noContentYet;

        public FlowDocument GetDocument()
        {
            return m_output.Document;
        }

        private string m_title;
        public string GetTitle()
        {
            return m_title;
        }

        [Match("/")]
        public void DocumentRoot(XPathNavigator context)
        {
            m_output = new FlowDocBuilder();
            ApplyTemplates("*");

            m_output.PutNextTextInNewParagraph();

            // Required by MTPS terms of use http://www.microsoft.com/info/cpyright.mspx
            m_output.AddText(
                "Copyright (c) 2004 Microsoft Corporation, One Microsoft Way, Redmond, Washington 98052-6399 U.S.A. All rights reserved.");
        }
        [Match("xhtml:div[@class='title']")]
        public void DocumentTitle(XPathNavigator context)
        {
            m_title = context.Value;
        }

        [Match("xhtml:div[@id='mainBody']")]
        public void MainBody(XPathNavigator context)
        {
            ApplyTemplates("*|text()");
        }

        [Match("xhtml:div[@class='alert']/xhtml:table")]
        public void AlertTable(XPathNavigator context)
        {
            m_output.StartTable("AlertTable");
            ApplyTemplates("xhtml:tr");
            m_output.EndTable();
            m_noContentYet = true;
        }

        [Match("xhtml:table")]
        public void Table(XPathNavigator context)
        {
            XPathNavigator node = context.SelectSingleNode("@class");
            m_output.StartTable(node == null ? null : node.Value);
            ApplyTemplates("xhtml:tr");
            m_output.EndTable();
            m_noContentYet = true;
        }

        [Match("xhtml:tr")]
        public void TableRow(XPathNavigator context)
        {
            m_output.AddTableRow();
            ApplyTemplates("xhtml:th|xhtml:td");
        }

        [Match("xhtml:th")]
        public void TableHeader(XPathNavigator context)
        {
            m_output.AddTableHeaderItem();
            ApplyTemplates("*|text()");
        }

        [Match("xhtml:td")]
        public void TableData(XPathNavigator context)
        {
            m_output.AddTableItem();
            ApplyTemplates("*|text()");
        }

        [Match("text()")]
        public void Text(XPathNavigator context)
        {
            string text = context.Value;
            text = text.Replace("\n", "").Replace("\t", "");
            if (m_noContentYet && text.Trim().Length == 0) return;

            m_noContentYet = false;
            m_output.AddText(text);
        }

        [Match("xhtml:p")]
        public void Paragraph(XPathNavigator context)
        {
            m_output.PutNextTextInNewParagraph();
            ApplyTemplates("*|text()");
        }

        [Match("mtps:CollapsibleArea")]
        public void CollapsibleArea(XPathNavigator context)
        {
            string heading = (string)context.Evaluate("string(@Title)");
            m_output.WriteTitle(heading);
            ApplyTemplates("*|text()");
        }


        [Match("xhtml:b")]
        public void Bold(XPathNavigator context)
        {
            m_output.StartBold();
            ApplyTemplates("*|text()");
            m_output.EndInline();
        }

        [Match("xhtml:i")]
        public void Italic(XPathNavigator context)
        {
            m_output.StartItalic();
            ApplyTemplates("*|text()");
            m_output.EndInline();
        }

        [Match("xhtml:br")]
        public void Break(XPathNavigator context)
        {
            m_output.LineBreak();
        }

        [Match("xhtml:ul")]
        public void UnorderedList(XPathNavigator context)
        {
            m_output.StartList(TextMarkerStyle.Disc);
            ApplyTemplates("xhtml:li");
            m_output.EndList();
            m_noContentYet = true;

        }

        [Match("xhtml:li")]
        public void ListItem(XPathNavigator context)
        {
            m_output.ListItem();
            m_noContentYet = true;
            ApplyTemplates("*|text()");
        }
        [Match("mtps:CodeSnippet")]
        public void CodeSnippet(XPathNavigator context)
        {
            //string snippetLanguage = (string)context.Evaluate("string(@Language)");
            string snippetDisplayLanguage = (string)context.Evaluate("string(@DisplayLanguage)");

            m_output.WriteSnippet(snippetDisplayLanguage, context.Value);
            m_noContentYet = true;

        }

        [Match("mtps:InstrumentedLink")]
        public void InstrumentedLink(XPathNavigator context)
        {
            string href = (string)context.Evaluate("string(@NavigateUrl)");

            m_output.StartHyperlink(href);
            ApplyTemplates("*|text()");
            m_output.EndInline();

        }

        [Match("xhtml:h1|xhtml:h2|xhtml:h3|xhtml:h4|xhtml:h5")]
        public void Headings(XPathNavigator context)
        {
            m_output.WriteTitle(context.Value);
            m_noContentYet = true;

        }

    }
}
